#!/usr/bin/python3
# -- coding: UTF-8 --
# -- Author: Blues --

import sys
from pyinotify import WatchManager, Notifier, ProcessEvent, IN_CREATE
import pyinotify
import ctypes
import time
import json
import psutil
from pydbus import SystemBus
import socket


class EventHandler(ProcessEvent):
    def __init__(self):
        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    """事件处理"""
    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        time = self.get_host_time
        name = self.get_host_name
        ip = self.get_host_ip
        log_message = "'time':'{}','hostname':'{}','ip':'{}','name':'storage-monitor','lv':'info','message':'{}'".format(
            time, name, ip, message)

        try:
            self.log.log_transmit('{' + log_message + '}')
        except Exception:
            print("log message error")

    def process_IN_CREATE(self, event):
        time.sleep(1)
        permission_so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")
        # 获取当前管控模式， gsettings因用户问题获取不精确

        try:
            self.log_transmit("存储设备初始化")
        except Exception:
            print("log message error")
            
        permission_so.setup_storage()
        sys.exit()


def FSMonitor(path: str) -> None:
    wm = WatchManager()
    # 添加监听事件类型
    mask = IN_CREATE
    notifier = Notifier(wm, EventHandler())
    wm.add_watch(path, mask, rec=True)
    while True:
        try:
            notifier.process_events()
            if notifier.check_events():
                notifier.read_events()
        except KeyboardInterrupt:
            notifier.stop()
            break


if __name__ == "__main__":
    FSMonitor("/media")
